==> sp-pkg_1.1.0.orig.tar.gz: sp_pkg-1.1.0/unit_tests/replace_rpm/test_functional.py <==
# SPDX-FileCopyrightText: StorPool <support@storpool.com>
# SPDX-License-Identifier: BSD-2-Clause
"""A test for the functionality of the replacerpm program."""

from __future__ import annotations

import dataclasses
import pathlib
import subprocess
import sys
import tempfile
import typing

import pytest

from sp_pkg.scan import rpm as s_rpm
from unit_tests import test_utils as tutil


if typing.TYPE_CHECKING:
    from typing import Final

    from sp_pkg import defs


@dataclasses.dataclass(frozen=True)
class BuiltConfig(tutil.BuiltRPMConfig):
    """Configuration settings and files prepared for testing."""

    t_cmdline: defs.PathList
    """The base invocation of the `replace-rpm` tool without any parameters."""

    t_cwd: pathlib.Path
    """The temporary directory to issue the commands from."""


@pytest.fixture
def built_cfg() -> BuiltConfig:
    """Set up the environment and stuff."""
    cfg, sp_hello = tutil.get_config()

    cwd: Final = pathlib.Path.cwd()
    if "PYTHONPATH" in cfg.subenv:
        cfg.subenv["PYTHONPATH"] = str(cwd) + ":" + cfg.subenv["PYTHONPATH"]
    else:
        cfg.subenv["PYTHONPATH"] = str(cwd)

    return BuiltConfig(
        t_cwd=cwd,
        t_cfg=cfg,
        t_cmdline=[sys.executable, "-m", "sp_pkg.replace_rpm"],
        t_built=s_rpm.scan_built(cfg, pathlib.Path(sp_hello)) if sp_hello else None,
    )


def test_help(built_cfg: BuiltConfig) -> None:
    """Make sure `replacerpm --help` outputs something."""
    lines: Final = built_cfg.t_cfg.check_output([*built_cfg.t_cmdline, "--help"]).splitlines()
    assert lines[0].startswith("usage: replacerpm ")


@pytest.mark.parametrize(
    "args",
    [
        [],
        ["/nonexistent"],
        ["/nonexistent", "/nonexistent"],
        ["/nonexistent", "/"],
    ],
)
def test_bad_args(built_cfg: BuiltConfig, args: defs.PathList) -> None:
    """`replacerpm` with no arguments should fail."""
    proc: Final = built_cfg.t_cfg.Popen(
        built_cfg.t_cmdline + args,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    p_out, p_err = proc.communicate()
    res: Final = proc.wait()
    assert not p_out
    assert p_err
    assert res


def test_packages(built_cfg: BuiltConfig) -> None:
    """Test actually copying packages into a repository."""
    if built_cfg.t_built is None:
        print("\nSkipping the packages test, no SP_HELLO.")
        return

    with tempfile.TemporaryDirectory() as tempd_name:
        tempd: Final = pathlib.Path(tempd_name)
        print(f"\nRunning a packages test in {tempd}")

        repodir: Final = tempd / "repo"
        tutil.create_empty_repo(built_cfg.t_cfg, repodir)

        before: Final = sorted(path for path in repodir.rglob("*") if path.is_file())
        print(f"- {len(before)} files in an empty repo")
        assert not any(path.name.endswith(".rpm") for path in before)

        print("- running replacerpm -N")
        built_cfg.t_cfg.check_call(
            [*built_cfg.t_cmdline, "-N", "--", repodir, built_cfg.t_built.path]
        )
        print("- making sure nothing has changed")
        after_mid: Final = sorted(path for path in repodir.rglob("*") if path.is_file())
        assert list(map(str, before)) == list(map(str, after_mid))

        print("- running replacerpm")
        built_cfg.t_cfg.check_call([*built_cfg.t_cmdline, "--", repodir, built_cfg.t_built.path])
        print("- making sure files have changed")
        after: Final = sorted(path for path in repodir.rglob("*") if path.is_file())
        assert list(map(str, before)) != list(map(str, after))

        print("- making sure all of the RPM files have been copied")
        assert (repodir / "SRPMS/Sources" / built_cfg.t_built.source.path.name).is_file()
        assert all(
            (repodir / "x86_64/Packages" / rpmf.path.name).is_file()
            for rpmf in built_cfg.t_built.binaries
        )

        print("- making sure the files count is correct")
        assert len(after) == len(before) + 1 + len(built_cfg.t_built.binaries)

        print("- running replacerpm -N again")
        built_cfg.t_cfg.check_call(
            [*built_cfg.t_cmdline, "-N", "--", repodir, built_cfg.t_built.path]
        )
        print("- making sure nothing has changed")
        again_mid: Final = sorted(path for path in repodir.rglob("*") if path.is_file())
        assert list(map(str, after)) == list(map(str, again_mid))

        print("- running replacerpm again")
        built_cfg.t_cfg.check_call([*built_cfg.t_cmdline, "--", repodir, built_cfg.t_built.path])
        print("- making sure files have changed")
        again: Final = sorted(path for path in repodir.rglob("*") if path.is_file())
        if again == after:
            print("- the files did not change, weird, but it happens")
        else:
            print("- making sure the files count has not changed")
            assert len(after) == len(again)
            print("- making sure only */repodata/* files have changed")
            assert all(
                path.parent.name == "repodata"
                for path in set(after).symmetric_difference(set(again))
            )

==> sp-pkg_1.2.0.orig.tar.gz: sp_pkg-1.2.0/unit_tests/replace_rpm/test_functional.py <==
# SPDX-FileCopyrightText: StorPool <support@storpool.com>
# SPDX-License-Identifier: BSD-2-Clause
"""A test for the functionality of the replacerpm program."""

from __future__ import annotations

import dataclasses
import pathlib
import subprocess  # noqa: S404
import sys
import tempfile
import typing

import pytest

from sp_pkg.scan import rpm as s_rpm
from unit_tests import test_utils as tutil


if typing.TYPE_CHECKING:
    from typing import Final

    from sp_pkg import defs


@dataclasses.dataclass(frozen=True)
class BuiltConfig(tutil.BuiltRPMConfig):
    """Configuration settings and files prepared for testing."""

    t_cmdline: defs.PathList
    """The base invocation of the `replace-rpm` tool without any parameters."""

    t_cwd: pathlib.Path
    """The temporary directory to issue the commands from."""


@pytest.fixture
def built_cfg() -> BuiltConfig:
    """Set up the environment and stuff."""
    cfg, sp_hello = tutil.get_config()

    cwd: Final = pathlib.Path.cwd()
    if "PYTHONPATH" in cfg.subenv:
        cfg.subenv["PYTHONPATH"] = str(cwd) + ":" + cfg.subenv["PYTHONPATH"]
    else:
        cfg.subenv["PYTHONPATH"] = str(cwd)

    return BuiltConfig(
        t_cwd=cwd,
        t_cfg=cfg,
        t_cmdline=[sys.executable, "-m", "sp_pkg.replace_rpm"],
        t_built=s_rpm.scan_built(cfg, pathlib.Path(sp_hello)) if sp_hello else None,
    )


def test_help(built_cfg: BuiltConfig) -> None:
    """Make sure `replacerpm --help` outputs something."""
    lines: Final = built_cfg.t_cfg.check_output([*built_cfg.t_cmdline, "--help"]).splitlines()
    assert lines[0].startswith("usage: replacerpm ")


@pytest.mark.parametrize(
    "args",
    [
        [],
        ["/nonexistent"],
        ["/nonexistent", "/nonexistent"],
        ["/nonexistent", "/"],
    ],
)
def test_bad_args(built_cfg: BuiltConfig, args: defs.PathList) -> None:
    """`replacerpm` with no arguments should fail."""
    proc: Final = built_cfg.t_cfg.Popen(
        built_cfg.t_cmdline + args,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    p_out, p_err = proc.communicate()
    res: Final = proc.wait()
    assert not p_out
    assert p_err
    assert res


def test_packages(built_cfg: BuiltConfig) -> None:
    """Test actually copying packages into a repository."""
    if built_cfg.t_built is None:
        print("\nSkipping the packages test, no SP_HELLO.")
        return

    with tempfile.TemporaryDirectory() as tempd_name:
        tempd: Final = pathlib.Path(tempd_name)
        print(f"\nRunning a packages test in {tempd}")

        repodir: Final = tempd / "repo"
        tutil.create_empty_repo(built_cfg.t_cfg, repodir)

        before: Final = sorted(path for path in repodir.rglob("*") if path.is_file())
        print(f"- {len(before)} files in an empty repo")
        assert not any(path.name.endswith(".rpm") for path in before)

        print("- running replacerpm -N")
        built_cfg.t_cfg.check_call(
            [*built_cfg.t_cmdline, "-N", "--", repodir, built_cfg.t_built.path],
        )
        print("- making sure nothing has changed")
        after_mid: Final = sorted(path for path in repodir.rglob("*") if path.is_file())
        assert list(map(str, before)) == list(map(str, after_mid))

        print("- running replacerpm")
        built_cfg.t_cfg.check_call([*built_cfg.t_cmdline, "--", repodir, built_cfg.t_built.path])
        print("- making sure files have changed")
        after: Final = sorted(path for path in repodir.rglob("*") if path.is_file())
        assert list(map(str, before)) != list(map(str, after))

        print("- making sure all of the RPM files have been copied")
        assert (repodir / "SRPMS/Sources" / built_cfg.t_built.source.path.name).is_file()
        assert all(
            (repodir / "x86_64/Packages" / rpmf.path.name).is_file()
            for rpmf in built_cfg.t_built.binaries
        )

        print("- making sure the files count is correct")
        assert len(after) == len(before) + 1 + len(built_cfg.t_built.binaries)

        print("- running replacerpm -N again")
        built_cfg.t_cfg.check_call(
            [*built_cfg.t_cmdline, "-N", "--", repodir, built_cfg.t_built.path],
        )
        print("- making sure nothing has changed")
        again_mid: Final = sorted(path for path in repodir.rglob("*") if path.is_file())
        assert list(map(str, after)) == list(map(str, again_mid))

        print("- running replacerpm again")
        built_cfg.t_cfg.check_call([*built_cfg.t_cmdline, "--", repodir, built_cfg.t_built.path])
        print("- making sure files have changed")
        again: Final = sorted(path for path in repodir.rglob("*") if path.is_file())
        if again == after:
            print("- the files did not change, weird, but it happens")
        else:
            print("- making sure the files count has not changed")
            assert len(after) == len(again)
            print("- making sure only */repodata/* files have changed")
            assert all(
                path.parent.name == "repodata"
                for path in set(after).symmetric_difference(set(again))
            )
